/*
struct OSPV_TV : OSPV_Generic {
  static os_typespec *get_os_typespec();
  XPVTV tv;
  OSPV_TV();
  virtual ~OSPV_TV();
  static void _boot(HV*);
};

struct OSPV_TC : OSPV_Cursor2 {
  static os_typespec *get_os_typespec();
  XPVTC tc;
};
*/
